@@ -14,10 +14,11 @@
 #
 
 package GnuPG::PublicKey;
+use Any::Moose;
 
-use strict;
+BEGIN { extends qw( GnuPG::PrimaryKey ) }
 
-use base qw( GnuPG::PrimaryKey );
+__PACKAGE__->meta->make_immutable;
 
 1;
 
