@@ -14,22 +14,33 @@
 #
 
 package GnuPG::UserId;
+use Any::Moose;
 
-use strict;
+has [qw( validity as_string )] => (
+    isa => 'Any',
+    is  => 'rw',
+);
 
-use Class::MethodMaker
-  get_set       => [ qw( validity as_string ) ],
-  list          => [ qw( signatures ) ],
-  new_hash_init => 'new';
+has signatures => (
+    isa       => 'ArrayRef',
+    is        => 'rw',
+    default   => sub { [] },
+);
+
+sub push_signatures {
+    my $self = shift;
+    push @{ $self->signatures }, @_;
+}
 
 # DEPRECATED
-sub user_id_string
-{
+sub user_id_string {
     my ( $self, $v ) = @_;
-    $self->as_string( $v ) if defined $v;
+    $self->as_string($v) if defined $v;
     return $self->as_string();
 }
 
+__PACKAGE__->meta->make_immutable;
+
 1;
 
 __END__
@@ -56,19 +67,12 @@ objects.
 =item new( I<%initialization_args> )
 
 This methods creates a new object.  The optional arguments are
-initialization of data members; the initialization is done
-in a manner according to the method created as described
-in L<Class::MethodMaker/"new_hash_init">.
+initialization of data members;
 
 =back
 
 =head1 OBJECT DATA MEMBERS
 
-Note that these data members are interacted with via object methods
-created using the methods described in L<Class::MethodMaker/"get_set">,
-L<Class::MethodMaker/"object">, or L<Class::MethodMaker/"list">.
-Please read there for more information.
-
 =over 4
 
 =item as_string
@@ -91,6 +95,5 @@ on this user id.
 =head1 SEE ALSO
 
 L<GnuPG::Signature>,
-L<Class::MethodMaker>
 
 =cut
