@@ -1,7 +1,7 @@
 #  ComparableKey.pm
 #    - comparable GnuPG::Key
 #
-#  Copyright (C) 2000 Frank J. Tobin <ftobin@uiuc.edu>
+#  Copyright (C) 2000 Frank J. Tobin <ftobin@cpan.org>
 #
 #  This module is free software; you can redistribute it and/or modify it
 #  under the same terms as Perl itself.
@@ -10,16 +10,15 @@
 #  but WITHOUT ANY WARRANTY; without even the implied warranty of
 #  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 #
-#  $Id: ComparableKey.pm,v 1.2 2001/04/30 00:09:26 ftobin Exp $
+#  $Id: ComparableKey.pm,v 1.4 2001/09/14 12:34:36 ftobin Exp $
 #
 
 package GnuPG::ComparableKey;
 
 use strict;
-use vars qw( @ISA );
-use GnuPG::Key;
+use GnuPG::Fingerprint;
 
-push @ISA, 'GnuPG::Key';
+use base qw( GnuPG::Key );
 
 sub compare
 {
